##############################################################################
#   
#   The BSD License
#   
#   Copyright (c) 2004, Metod.ru
#   All rights reserved.
#   
#   Redistribution and use in source and binary forms, with or without modification,
#   are permitted provided that the following conditions are met:
#   
#   	1. Redistributions of source code must retain the above
#   	   copyright notice, this list of conditions and the following
#   	   disclaimer.
#   
#   	2. Redistributions in binary form must reproduce the above
#   	   copyright notice, this list of conditions and the following
#   	   disclaimer in the documentation and/or other materials
#   	   provided with the distribution.
#   
#   	3. Neither the name of the Metod.ru nor the names of its
#   	   contributors may be used to endorse or promote products derived
#   	   from this software without specific prior written permission.
#   
#   THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS"
#   AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE
#   ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR CONTRIBUTORS BE
#   LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR
#   CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF
#   SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS
#   INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN
#   CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE)
#   ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
#   POSSIBILITY OF SUCH DAMAGE.
#   
#   Lavrinenko Sergey    lavrinenko@metod.ru
#   Kondiain Vladislav   fei@metod.ru
#   Sokolova Antonina    sokolova@metod.ru
#   
#   Metod.ru             http://www.metod.ru/
#   
##############################################################################

""" Anti bot Form Protection """

import os
import Globals
from Globals import InitializeClass, DTMLFile
from OFS.SimpleItem import SimpleItem
from OFS.SimpleItem import Item

from OFS.PropertyManager import PropertyManager
from AccessControl.Role import RoleManager
from AccessControl import ClassSecurityInfo

from PIL import Image, ImageDraw, ImageFont
from whrandom import randrange
import cStringIO




class MFormProtection(SimpleItem, PropertyManager ):
	""" define class MFormProtection """

	meta_type = 'MFormProtection'
	manage_propertiesForm = DTMLFile('dtml/manage_propertiesForm',globals())

	_properties = (
		{'id':'numDigits', 'type': 'int', 'mode': 'w'},
		{'id':'listUid', 'type': 'int', 'mode': 'w'}
			)

	manage_options = PropertyManager.manage_options \
		+ RoleManager.manage_options \
		+ Item.manage_options

	security=ClassSecurityInfo()


	def __init__(self, id, REQUEST):
		"initialise a new instance of MFormProtection"
		self.id = id
		self.numDigits = 4
		self.listUid = []


	def draw_(self, string, width=40, height=15, bgcolor=0x663300, fontname='courR10.pil', fontcolor=0xc4c4c4, noise=0):
		""" draw string on image, using PIL """
		image = Image.new('RGB',(width,height),bgcolor)
		draw  = ImageDraw.Draw(image)
		font  = ImageFont.load('%s%s'%(os.path.join(Globals.package_home(globals()), 'PILFonts/'),fontname))
		draw.text((2, 0), str(string), font=font, fill=fontcolor)

		if noise==1:
			for i in range(1,10,1):		# draw random 10 points
				x = randrange(3,width,1)
				y = randrange(3,height,1)
				draw.point((x,y),fill=0xc4c4c4)

			x = randrange(1,width-5,1)	# draw random line
			y = randrange(1,height-3,1)
			draw.line([(x,y),(x+9,y+5)],fill=0xc4c4c4)

		io = cStringIO.StringIO()
		image.save(io, 'gif')
		io.seek(0)
		del draw

		return self.REQUEST.RESPONSE.write(io.getvalue())


	def digits_gif(self):
		""" draw() wrapper """
		return self.draw()

	def draw(self):
		""" call functions draw_(), randomDigits(),
		    insert uid entry into property obj FormProtection,
		    delete uid entry that inserted 20 minute later
		"""
		s = self.randomDigits(self.numDigits)
		w = len(s)*10
		newArray = []
		if len(self.listUid)>0:
			ms = (1.0/24/60)*20 # 20 minutes
			for i in self.listUid:
				try:
					date = i.split('|')[-1]
					if self.ZopeTime() - self.ZopeTime(date) > ms:
						pass
					else:
						newArray.append(i)
				except:
					#print "error ZopeTime %s" % i
					pass

		newArray.append('%s|%s|%s'%(self.REQUEST.SESSION.getName(),s,self.ZopeTime()))
		self.manage_changeProperties({'listUid': newArray})

		return self.draw_(s,width=w,noise=1)


	def randomDigits(self, n):
		""" return random 4 digits """
		rnd = randrange(1, 10**n-1, 3)
		fmt = "%%0%sd" % n
		return fmt % rnd


	def checkUid(self, codeDigits):
		""" check if string(sessionName|uidFromForm) exist in property FormProtection """
		if len(codeDigits)==self.numDigits:
			struid = str(self.REQUEST.SESSION.getName()) + '|' + str(codeDigits)

			for i in self.listUid:
				if i[:len(struid)]==struid:
					return 1


	security.declareProtected('Edit FormProtection', 'manage_changeMFormProtection')
	def manage_changeMFormProtection(self, REQUEST=None, RESPONSE=None):
		""" change properties MFormProtection """
		self.numDigits = getattr(REQUEST,'numDigits',0)
		self.listUid   = getattr(REQUEST,'listUid','')

		if RESPONSE:
			RESPONSE.redirect(self.absolute_url() + '/'+self.id+'/manage_propertiesForm')




def manage_addMFormProtection(self, id, REQUEST=None, RESPONSE=None):
	""" Add a new MFormProtection to current ObjectManager """
	mfp = MFormProtection(id, REQUEST)
	self._setObject(id, mfp)
	mf  = self._getOb(id)

	if RESPONSE:
		RESPONSE.redirect(self.absolute_url() + '/'+id+'/manage_propertiesForm')


def manage_addMFormProtectionForm(self):
	""" form """
	return """<html>
		<head>
		<title>Add MFormProtection</title>
		</head>
		<body>
		id:<br>
		<form name="form" action="manage_addMFormProtection"><br>
			<input type="text" name="id" value="FormProtection"><br>
			<input type="submit" value="add">
		</form>
		</body>
		</html>"""


Globals.InitializeClass(MFormProtection)